/*****************************************************************************************
**
** vEnglish - French-English Dictionary and more !
** Contact : vin.fouache@free.fr
**
** Copyright (C) 2008  Vincent Fouache
**
** This file is part of vEnglish.
**
** vEnglish is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>
**
***************************************************************************************/


/* ********** ********** ********** ********** ********** ********* *\
*
*               Dicos.cc
*               --------
*
*   Auteur : Vincent Fouache
*   Date de création : 30 Octobre 2008
*   Dernières modifications : 01/11/08
*
*   Description : Implémentation de la classe Dicos
*
\* ********** ********** ********** ********** ********** ********* */

#include "Dicos.hh"


Dicos::Dicos()
{
    separation = ':';
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

int Dicos::fileToDicos()
{
    int erreur = 0;

    /* **********************************************************************
        Ouverture du fichier
    ********************************************************************** */

    QFile file("dico.txt");
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        erreur = 1;
    else
    {
        // Initialisation des lists
        fr.clear();
        en.clear();

        /* **********************************************************************
            Lecture du fichier, insertion des valeurs ds les lists
        ********************************************************************** */

        QTextStream in(&file);
        while (!in.atEnd())
        {
            // On lit une ligne qu'on stocke ds line.
            QString line = in.readLine();

            // On récupère l'expression française puis l'expression anglaise
            QString strFR = line.section(separation, 0, 0);
            QString strEN = line.section(separation, 1, 1);

            // Si il manque un terme, par exemple "oui:" ou ":yes"
            if (strFR == "" || strEN == "")
                erreur = 2;

            // Ajout des 2 termes ds les lists correspondantes
            fr.append(strFR);
            en.append(strEN);
        }

        // On ferme le fichier.
        file.close();
    }

    return erreur;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

QString Dicos::getFr(const int i) const
{
    return fr[i];
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

QString Dicos::getEn(const int i) const
{
    return en[i];
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

QChar Dicos::getSeparation() const
{
    return separation;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

int Dicos::taille() const
{
    // Je prends la taille du dico fr pour compter le nb de termes du dico
    // mais j'aurais aussi bien pu prendre celle du dico en ;)

    return fr.size();
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Modifie ou ajoute une ligne dans un des dicos
bool Dicos::setTrad(const int i, const QString &str, const Langue lang)
{
    bool pasErreur = true;

    // Si le terme ajouté ou modifié contient le caractère de séparation utilisé dans le fichier
    if (str.contains(separation))
        pasErreur = false;
    else
    {
        // On ne se sert pas de taille()
        // Il faut récupérer la taille pour chaque langue.
        // En effet, lors d'un ajout d'une traduction, c'est d'abord le dico de la colonne gauche qui est modifié puis celui de droite
        // Donc il y a un moment où les dicos n'ont pas la même taille...
        int taille;
        if (lang == FR)
            taille = fr.size();
        else
            taille = en.size();

        // Si i est égal à la taille de la liste (qui commence à l'indice 0)
        // Alors on ajoute une ligne
        if (i == taille)
        {
            if (lang == FR)
                fr.append(str);
            else
                en.append(str);
        }
        // Sinon on modifie la ligne demandée.
        else if (i < taille)
        {
            if (lang == FR)
                fr[i] = str;
            else
                en[i] = str;
        }
    }

    return pasErreur;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void Dicos::delTrad(const int i, const Langue lang)
{
    if (lang == FR)
        fr.removeAt(i);
    else
        en.removeAt(i);
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void Dicos::toFile()
{
    // Ouverture du fichier
    QFile file("dico.txt");
    // Si le fichier n'existe pas, il sera créé
    file.open(QIODevice::WriteOnly | QIODevice::Text);

    QTextStream stream(&file);

    // Ecriture du fichier
    file.reset();
    for (int i = 0 ; i < taille() ; i++)
        stream << fr[i] << separation << en[i] << endl;

    // Fermeture du fichier
    file.close();
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

void Dicos::trier(const Tri tri = CROISSANT, const bool casse = false)
{
    QStringList dicoComplet;

    // Remplissage de la list dicoComplet avec les termes affichés actuellement
    // i.e. les termes des lists et non ceux du fichier
    for (int i = 0 ; i < taille() ; i++)
        dicoComplet.append(fr[i] + separation + en[i]);


    // Tri alphabétique
    if (casse == false)
    {
        QMap<QString, QString> map;
        foreach (QString str, dicoComplet)
            map.insert(str.toLower(), str);
        dicoComplet = map.values();
    }
    else
        dicoComplet.sort();


    // On remet à zéro les 2 lists.
    fr.clear();
    en.clear();

    if (tri == CROISSANT)
    {
        for (int i = 0 ; i < dicoComplet.size() ; i++)
        {
            // Ajout des 2 termes (séparés) ds les lists correspondantes
            fr.append(dicoComplet[i].section(separation, 0, 0));
            en.append(dicoComplet[i].section(separation, 1, 1));
        }
    }
    else
    {
        // Pour trier ds l'ordre décroissant
        // il suffit de faire la boucle ds l'ordre inverse :p
        for (int i = dicoComplet.size() - 1 ; i >= 0 ; i--)
        {
            // Ajout des 2 termes (séparés) ds les lists correspondantes
            fr.append(dicoComplet[i].section(separation, 0, 0));
            en.append(dicoComplet[i].section(separation, 1, 1));
        }
    }
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//*********************************************************************************************//
/////////////////////////////////////////////////////////////////////////////////////////////////

int Dicos::indexOf(const QRegExp &re, const int i, const Langue lang)
{
    int res;
    if (lang == FR)
        res = fr.indexOf(re, i);
    else // lang == EN
        res = en.indexOf(re, i);
    return res;
}
